==> tecfy_core_package-0.1.102.tar.gz: lib/views/components/entity/mark_image_property.dart <==
part of tecfy_core_client_package;

class MarkImagePropertyComponenet extends StatefulWidget {
  final String initalValue;
  final String label;
  final bool isRequired;
  final String fileId;
  final void Function(String) onChanged;
  MarkImagePropertyComponenet(
      {this.initalValue,
      this.label,
      this.isRequired,
      this.onChanged,
      this.fileId});
  @override
  _MarkImagePropertyComponenetState createState() =>
      _MarkImagePropertyComponenetState();
}

class _MarkImagePropertyComponenetState
    extends State<MarkImagePropertyComponenet> {
  List<Offset> points;
  bool marking = false;
  double x = 0, y = 0;
  double scale = 1;
  ui.Image image;
  // static int imageWidth = 5346;
  // static int imageHeight = 4257;
  double pScale = 4257 / 500; // used to convert the x,y to pixel
  @override
  initState() {
    if (widget.initalValue == null || widget.initalValue.isEmpty) {
      points = [];
    } else {
      points = json
          .decode(widget.initalValue)
          .map<Offset>((c) =>
              Offset((c[0] / pScale).toDouble(), (c[1] / pScale).toDouble()))
          .toList();
    }
    //points = [Offset(258/pScale,1055/pScale)];
    super.initState();
  }

  Future<ui.Image> loadImage(List<int> img) async {
    final Completer<ui.Image> completer = new Completer();
    ui.decodeImageFromList(img, (ui.Image img) {
      return completer.complete(img);
    });
    return completer.future;
  }

  onChanged() {
    var r = json.encode(points
        .map((e) => [(e.dx * pScale).toInt(), (e.dy * pScale).toInt()])
        .toList());
    widget.onChanged(r);
  }

  @override
  Widget build(BuildContext context) {
    return Container(
        child: Stack(
          children: [
            Transform.scale(
              origin: Offset(x, y),
              scale: scale,
              child: GestureDetector(
                onPanUpdate: (details) => setState(() {
                  if (scale > 1) {
                    x -= details.delta.dx;
                    y -= details.delta.dy;
                  } else {
                    x += details.delta.dx;
                    y += details.delta.dy;
                  }
                }),
                onTapDown: (p) {
                  if (!marking) return;
                  points.add(p.localPosition);
                  onChanged();
                  setState(() {});
                },
                child: Stack(children: [
                  Align(
                      alignment: Alignment.topLeft,
                      child: Image.network(
                          'https://watanaqar.com/sites/default/files/2018-08/%D8%A7%D9%84%D8%A3%D9%85%D8%B1%D8%A7%D8%A1%20%D8%A7%D9%84%D9%87%D9%81%D9%88%D9%81.jpg',
                          alignment: Alignment.topLeft,
                          width: double.infinity,
                          height: double.infinity,
                          fit: BoxFit.fitHeight)),
                  Container(
                      width: double.infinity,
                      height: double.infinity,
                      child: CustomPaint(
                          child: Visibility(
                              visible: false,
                              child: Image.network(
                                  'https://watanaqar.com/sites/default/files/2018-08/%D8%A7%D9%84%D8%A3%D9%85%D8%B1%D8%A7%D8%A1%20%D8%A7%D9%84%D9%87%D9%81%D9%88%D9%81.jpg',
                                  alignment: Alignment.topLeft,
                                  fit: BoxFit.fitHeight)),
                          painter: PixelGridPainter(
                              data: points, image: image, scale: scale)))
                ]),
              ),
            ),
            Align(
                alignment: Alignment.topRight,
                child: Padding(
                    padding: EdgeInsets.all(5),
                    child: Column(
                      children: [
                        CircleAvatar(
                            backgroundColor: ThemeService.mainColor,
                            child: IconButton(
                                icon: Icon(Icons.clear),
                                tooltip: 'Clear',
                                color: Colors.red,
                                onPressed: () {
                                  points = [];
                                  setState(() {});
                                })),
                        SizedBox(height: 5),
                        CircleAvatar(
                            backgroundColor: ThemeService.mainColor,
                            child: IconButton(
                                icon: Icon(marking ? Icons.done : Icons.edit),
                                tooltip: marking ? 'Done' : 'Start Marke',
                                color:
                                    marking ? Colors.green : Colors.greenAccent,
                                onPressed: () {
                                  setState(() {
                                    marking = !marking;
                                  });
                                })),
                        SizedBox(height: 5),
                        CircleAvatar(
                            backgroundColor: ThemeService.mainColor,
                            child: IconButton(
                                icon: Icon(Icons.zoom_in),
                                tooltip: 'Zoom In',
                                color: ThemeService.textOnMainColor,
                                onPressed: () => setState(() => scale *= 1.2))),
                        SizedBox(height: 5),
                        CircleAvatar(
                            backgroundColor: ThemeService.mainColor,
                            child: IconButton(
                                icon: Icon(Icons.zoom_out),
                                tooltip: 'Zoom Out',
                                color: ThemeService.textOnMainColor,
                                onPressed: () =>
                                    setState(() => scale -= (scale * 0.2)))),
                      ],
                    )))
          ],
        ),
        constraints: BoxConstraints(maxHeight: 500));
  }
}

class PixelGridPainter extends CustomPainter {
  final List<Offset> data;
  final ui.Image image;
  final double scale;
  PixelGridPainter({this.data, this.image, this.scale});

  @override
  void paint(Canvas canvas, Size size) {
    // final double boxSide =
    //     min(size.width / rows[0].length, size.height / rows.length);
    //canvas.scale(100,100);

    final Paint paint0 = Paint();
    paint0.color = Colors.black;
    paint0.strokeWidth = 0.5;
    paint0.style = PaintingStyle.stroke;

    if (image != null) canvas.drawImage(image, Offset(0, 0), paint0);

    canvas.drawLine(Offset(0, 0), Offset(0, size.height), paint0);
    canvas.drawLine(
        Offset(0, size.height), Offset(size.width, size.height), paint0);
    canvas.drawLine(
        Offset(size.width, size.height), Offset(size.width, 0), paint0);
    canvas.drawLine(Offset(size.width, 0), Offset(0, 0), paint0);

    if (data.length <= 0) return;

    final Paint paint = Paint();
    paint.color = Colors.yellowAccent;
    paint.style = PaintingStyle.fill;
    paint.strokeWidth = 2 / scale;
    canvas.drawCircle(data[0], 5 / scale, paint);
    for (var i = 1; i < data.length; i++) {
      canvas.drawCircle(data[i], 5 / scale, paint);
      canvas.drawLine(data[i - 1], data[i], paint);
    }
    canvas.drawLine(data[data.length - 1], data[0], paint);
  }

  @override
  bool shouldRepaint(PixelGridPainter old) {
    return true;
  }
}

==> tecfy_core_package-1.0.5.tar.gz: lib/views/components/entity/mark_image_property.dart <==
part of tecfy_core_client_package;

class MarkImagePropertyComponenet extends StatefulWidget {
  final String? initalValue;
  final String? label;
  final bool isRequired;
  final String? fileId;
  final void Function(String)? onChanged;
  MarkImagePropertyComponenet(
      {this.initalValue,
      this.label,
      this.isRequired = false,
      this.onChanged,
      this.fileId});
  @override
  _MarkImagePropertyComponenetState createState() =>
      _MarkImagePropertyComponenetState();
}

class _MarkImagePropertyComponenetState
    extends State<MarkImagePropertyComponenet> {
  late List<Offset> points;
  bool marking = false;
  double x = 0, y = 0;
  double scale = 1;
  ui.Image? image;
  // static int imageWidth = 5346;
  // static int imageHeight = 4257;
  double pScale = 4257 / 500; // used to convert the x,y to pixel
  @override
  initState() {
    if (widget.initalValue == null || widget.initalValue!.isEmpty) {
      points = [];
    } else {
      points = json
          .decode(widget.initalValue!)
          .map<Offset>((c) =>
              Offset((c[0] / pScale).toDouble(), (c[1] / pScale).toDouble()))
          .toList();
    }
    //points = [Offset(258/pScale,1055/pScale)];
    super.initState();
  }

  Future<ui.Image> loadImage(Uint8List img) async {
    final Completer<ui.Image> completer = new Completer();
    ui.decodeImageFromList(img, (ui.Image img) {
      return completer.complete(img);
    });
    return completer.future;
  }

  onChanged() {
    var r = json.encode(points
        .map((e) => [(e.dx * pScale).toInt(), (e.dy * pScale).toInt()])
        .toList());
    widget.onChanged?.call(r);
  }

  @override
  Widget build(BuildContext context) {
    return Container(
        child: Stack(
          children: [
            Transform.scale(
              origin: Offset(x, y),
              scale: scale,
              child: GestureDetector(
                onPanUpdate: (details) => setState(() {
                  if (scale > 1) {
                    x -= details.delta.dx;
                    y -= details.delta.dy;
                  } else {
                    x += details.delta.dx;
                    y += details.delta.dy;
                  }
                }),
                onTapDown: (p) {
                  if (!marking) return;
                  points.add(p.localPosition);
                  onChanged();
                  setState(() {});
                },
                child: Stack(children: [
                  Align(
                      alignment: Alignment.topLeft,
                      child: Image.network(
                          'https://watanaqar.com/sites/default/files/2018-08/%D8%A7%D9%84%D8%A3%D9%85%D8%B1%D8%A7%D8%A1%20%D8%A7%D9%84%D9%87%D9%81%D9%88%D9%81.jpg',
                          alignment: Alignment.topLeft,
                          width: double.infinity,
                          height: double.infinity,
                          fit: BoxFit.fitHeight)),
                  Container(
                      width: double.infinity,
                      height: double.infinity,
                      child: CustomPaint(
                          child: Visibility(
                              visible: false,
                              child: Image.network(
                                  'https://watanaqar.com/sites/default/files/2018-08/%D8%A7%D9%84%D8%A3%D9%85%D8%B1%D8%A7%D8%A1%20%D8%A7%D9%84%D9%87%D9%81%D9%88%D9%81.jpg',
                                  alignment: Alignment.topLeft,
                                  fit: BoxFit.fitHeight)),
                          painter: PixelGridPainter(
                              data: points, image: image, scale: scale)))
                ]),
              ),
            ),
            Align(
                alignment: Alignment.topRight,
                child: Padding(
                    padding: EdgeInsets.all(5),
                    child: Column(
                      children: [
                        CircleAvatar(
                            backgroundColor: ThemeService.mainColor,
                            child: IconButton(
                                icon: Icon(Icons.clear),
                                tooltip: 'Clear',
                                color: Colors.red,
                                onPressed: () {
                                  points = [];
                                  setState(() {});
                                })),
                        SizedBox(height: 5),
                        CircleAvatar(
                            backgroundColor: ThemeService.mainColor,
                            child: IconButton(
                                icon: Icon(marking ? Icons.done : Icons.edit),
                                tooltip: marking ? 'Done' : 'Start Marke',
                                color:
                                    marking ? Colors.green : Colors.greenAccent,
                                onPressed: () {
                                  setState(() {
                                    marking = !marking;
                                  });
                                })),
                        SizedBox(height: 5),
                        CircleAvatar(
                            backgroundColor: ThemeService.mainColor,
                            child: IconButton(
                                icon: Icon(Icons.zoom_in),
                                tooltip: 'Zoom In',
                                color: ThemeService.textOnMainColor,
                                onPressed: () => setState(() => scale *= 1.2))),
                        SizedBox(height: 5),
                        CircleAvatar(
                            backgroundColor: ThemeService.mainColor,
                            child: IconButton(
                                icon: Icon(Icons.zoom_out),
                                tooltip: 'Zoom Out',
                                color: ThemeService.textOnMainColor,
                                onPressed: () =>
                                    setState(() => scale -= (scale * 0.2)))),
                      ],
                    )))
          ],
        ),
        constraints: BoxConstraints(maxHeight: 500));
  }
}

class PixelGridPainter extends CustomPainter {
  final List<Offset> data;
  final ui.Image? image;
  final double scale;
  PixelGridPainter({required this.data, this.image, required this.scale});

  @override
  void paint(Canvas canvas, Size size) {
    // final double boxSide =
    //     min(size.width / rows[0].length, size.height / rows.length);
    //canvas.scale(100,100);

    final Paint paint0 = Paint();
    paint0.color = Colors.black;
    paint0.strokeWidth = 0.5;
    paint0.style = PaintingStyle.stroke;

    if (image != null) canvas.drawImage(image!, Offset(0, 0), paint0);

    canvas.drawLine(Offset(0, 0), Offset(0, size.height), paint0);
    canvas.drawLine(
        Offset(0, size.height), Offset(size.width, size.height), paint0);
    canvas.drawLine(
        Offset(size.width, size.height), Offset(size.width, 0), paint0);
    canvas.drawLine(Offset(size.width, 0), Offset(0, 0), paint0);

    if (data.length <= 0) return;

    final Paint paint = Paint();
    paint.color = Colors.yellowAccent;
    paint.style = PaintingStyle.fill;
    paint.strokeWidth = 2 / scale;
    canvas.drawCircle(data[0], 5 / scale, paint);
    for (var i = 1; i < data.length; i++) {
      canvas.drawCircle(data[i], 5 / scale, paint);
      canvas.drawLine(data[i - 1], data[i], paint);
    }
    canvas.drawLine(data[data.length - 1], data[0], paint);
  }

  @override
  bool shouldRepaint(PixelGridPainter old) {
    return true;
  }
}
